﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ServiceModel.Samples.Discovery;
using System.Collections.ObjectModel;
using System.ServiceModel;

namespace PokerService
{
    /// <summary>
    /// A helper class which can be used to discover poker games
    /// </summary>
    public class DiscoveryHelper
    {
        /// <summary>
        /// Tries to discover game servers in the LAN. This method blocks for at least two seconds.
        /// </summary>
        /// <returns>
        /// A collection of server responses which were accepted.
        /// </returns>
        public IEnumerable<ServiceLocation> Discover()
        {
            // create a new service finder 
            ServiceFinder finder = new ServiceFinder();
            // probe the network
            ReadOnlyCollection<ServiceProperties> found = finder.Probe(new MatchCriteria(typeof(IPokerHost)), TimeSpan.FromSeconds(2), int.MaxValue);
            // the binding which is used to connect to each server
            NetTcpBinding binding = new NetTcpBinding(SecurityMode.None);
            // the result array
            ServiceLocation[] location = new ServiceLocation[found.Count];
            int i = 0;
            foreach (ServiceProperties prop in found)
            {
                // set the current result value
                location[i] = new ServiceLocation(prop.EndpointAddress);
                // try to aquire server details
                try
                {
                    // create a channel to get the server details
                    IPokerHost host = ChannelFactory<IPokerHost>.CreateChannel(binding, prop.EndpointAddress);
                    using (host as IDisposable)
                    {
                        location[i].ServerDetails = host.GetServerDetails();
                    }
                }
                catch
                {
                    // ignore misbehaved servers, return them as well
                }
                ++i;
            }
            return location;
        }
    }
}
